@@ -34,6 +34,7 @@ guac_common_ssh_user* guac_common_ssh_create_user(const char* username) {
     user->username = guac_strdup(username);
     user->password = NULL;
     user->private_key = NULL;
+    user->public_key = NULL;
 
     return user;
 
@@ -48,6 +49,7 @@ void guac_common_ssh_destroy_user(guac_common_ssh_user* user) {
     /* Free all other data */
     guac_mem_free(user->password);
     guac_mem_free(user->username);
+    guac_mem_free(user->public_key);
     guac_mem_free(user);
 
 }
@@ -83,3 +85,15 @@ int guac_common_ssh_user_import_key(guac_common_ssh_user* user,
 
 }
 
+int guac_common_ssh_user_import_public_key(guac_common_ssh_user* user,
+        char* public_key) {
+
+    /* Free existing public key, if present */
+    guac_mem_free(user->public_key);
+    user->public_key = guac_strdup(public_key);
+
+    /* Fail if key could not be read */
+    return user->public_key == NULL;
+
+}
+
