@@ -22,6 +22,7 @@
 #include "guacamole/mem.h"
 
 #include <stddef.h>
+#include <stdio.h>
 #include <string.h>
 
 /**
@@ -44,6 +45,20 @@
  */
 #define REMAINING(n, length) (((n) < (length)) ? 0 : ((n) - (length)))
 
+int guac_itoa(char* restrict dest, unsigned int integer) {
+
+    /* Determine size of string. */
+    int str_size = snprintf(dest, 0, "%i", integer);
+
+    /* If an error occurs, just return that and skip the conversion. */
+    if (str_size < 0)
+        return str_size;
+
+    /* Do the conversion and return. */
+    return snprintf(dest, (str_size + 1), "%i", integer);
+
+}
+
 size_t guac_strlcpy(char* restrict dest, const char* restrict src, size_t n) {
 
 #ifdef HAVE_STRLCPY
@@ -115,7 +130,7 @@ char* guac_strnstr(const char *haystack, const char *needle, size_t len) {
 
 }
 
-char* guac_strdup(const char* str) {
+char* guac_strndup(const char* str, size_t n) {
 
     /* Return NULL if no string provided */
     if (str == NULL)
@@ -124,18 +139,30 @@ char* guac_strdup(const char* str) {
     /* Do not attempt to duplicate if the length is somehow magically so
      * obscenely large that it will not be possible to add a null terminator */
     size_t length;
-    if (guac_mem_ckd_add(&length, strlen(str), 1))
+    size_t length_to_copy = strnlen(str, n);
+    if (guac_mem_ckd_add(&length, length_to_copy, 1))
         return NULL;
 
-    /* Otherwise just copy to a new string in same manner as strdup() */
-    void* new_str = guac_mem_alloc(length);
-    if (new_str != NULL)
-        memcpy(new_str, str, length);
+    /* Otherwise just copy to a new string in same manner as strndup() */
+    char* new_str = (char*)guac_mem_alloc(length);
+    if (new_str != NULL) {
+        memcpy(new_str, str, length_to_copy);
+        new_str[length_to_copy] = '\0';
+    }
 
     return new_str;
 
 }
 
+char* guac_strdup(const char* str) {
+
+    /* Return NULL if no string provided */
+    if (str == NULL)
+        return NULL;
+
+    return guac_strndup(str, strlen(str));
+}
+
 size_t guac_strljoin(char* restrict dest, const char* restrict const* elements,
         int nmemb, const char* restrict delim, size_t n) {
 
@@ -159,4 +186,3 @@ size_t guac_strljoin(char* restrict dest, const char* restrict const* elements,
     return length;
 
 }
-
