@@ -18,6 +18,7 @@
  */
 
 #include <CUnit/CUnit.h>
+#include <guacamole/mem.h>
 #include <guacamole/string.h>
 
 #include <stdlib.h>
@@ -46,4 +47,6 @@ void test_string__strdup() {
     CU_ASSERT_STRING_EQUAL(dest_string, "Mashing avocados.");
     CU_ASSERT_PTR_NULL(null_copy);
 
-}
\ No newline at end of file
+    guac_mem_free(dest_string);
+
+}
