@@ -107,13 +107,11 @@ guac_stream* guac_user_alloc_stream(guac_user* user) {
     guac_stream* allocd_stream;
     int stream_index;
 
-    /* Refuse to allocate beyond maximum */
-    if (user->__stream_pool->active == GUAC_USER_MAX_STREAMS)
+    /* Allocate stream, but refuse to allocate beyond maximum */
+    stream_index = guac_pool_next_int_below(user->__stream_pool, GUAC_USER_MAX_STREAMS);
+    if (stream_index < 0)
         return NULL;
 
-    /* Allocate stream */
-    stream_index = guac_pool_next_int(user->__stream_pool);
-
     /* Initialize stream with even index (odd indices are client-level) */
     allocd_stream = &(user->__output_streams[stream_index]);
     allocd_stream->index = stream_index * 2;
@@ -128,12 +126,13 @@ guac_stream* guac_user_alloc_stream(guac_user* user) {
 
 void guac_user_free_stream(guac_user* user, guac_stream* stream) {
 
-    /* Release index to pool */
-    guac_pool_free_int(user->__stream_pool, stream->index / 2);
-
     /* Mark stream as closed */
+    int freed_index = stream->index;
     stream->index = GUAC_USER_CLOSED_STREAM_INDEX;
 
+    /* Release index to pool */
+    guac_pool_free_int(user->__stream_pool, freed_index / 2);
+
 }
 
 guac_object* guac_user_alloc_object(guac_user* user) {
@@ -141,13 +140,11 @@ guac_object* guac_user_alloc_object(guac_user* user) {
     guac_object* allocd_object;
     int object_index;
 
-    /* Refuse to allocate beyond maximum */
-    if (user->__object_pool->active == GUAC_USER_MAX_OBJECTS)
+    /* Allocate object, but refuse to allocate beyond maximum */
+    object_index = guac_pool_next_int_below(user->__object_pool, GUAC_USER_MAX_OBJECTS);
+    if (object_index < 0)
         return NULL;
 
-    /* Allocate object */
-    object_index = guac_pool_next_int(user->__object_pool);
-
     /* Initialize object */
     allocd_object = &(user->__objects[object_index]);
     allocd_object->index = object_index;
@@ -161,12 +158,13 @@ guac_object* guac_user_alloc_object(guac_user* user) {
 
 void guac_user_free_object(guac_user* user, guac_object* object) {
 
-    /* Release index to pool */
-    guac_pool_free_int(user->__object_pool, object->index);
-
     /* Mark object as undefined */
+    int freed_index = object->index;
     object->index = GUAC_USER_UNDEFINED_OBJECT_INDEX;
 
+    /* Release index to pool */
+    guac_pool_free_int(user->__object_pool, freed_index);
+
 }
 
 int guac_user_handle_instruction(guac_user* user, const char* opcode, int argc, char** argv) {
