@@ -107,7 +107,8 @@ static void guac_rdp_send_key_event(guac_rdp_client* rdp_client,
 
     /* Send actual key */
     pthread_mutex_lock(&(rdp_client->message_lock));
-    rdp_inst->input->KeyboardEvent(rdp_inst->input, flags | pressed_flags, scancode);
+    GUAC_RDP_CONTEXT(rdp_inst)->input->KeyboardEvent(
+            GUAC_RDP_CONTEXT(rdp_inst)->input, flags | pressed_flags, scancode);
     pthread_mutex_unlock(&(rdp_client->message_lock));
 
 }
@@ -136,7 +137,8 @@ static void guac_rdp_send_unicode_event(guac_rdp_client* rdp_client,
 
     /* Send Unicode event */
     pthread_mutex_lock(&(rdp_client->message_lock));
-    rdp_inst->input->UnicodeKeyboardEvent(rdp_inst->input, 0, codepoint);
+    GUAC_RDP_CONTEXT(rdp_inst)->input->UnicodeKeyboardEvent(
+            GUAC_RDP_CONTEXT(rdp_inst)->input, 0, codepoint);
     pthread_mutex_unlock(&(rdp_client->message_lock));
 
 }
@@ -165,7 +167,8 @@ static void guac_rdp_send_synchronize_event(guac_rdp_client* rdp_client,
 
     /* Synchronize lock key states */
     pthread_mutex_lock(&(rdp_client->message_lock));
-    rdp_inst->input->SynchronizeEvent(rdp_inst->input, flags);
+    GUAC_RDP_CONTEXT(rdp_inst)->input->SynchronizeEvent(
+            GUAC_RDP_CONTEXT(rdp_inst)->input, flags);
     pthread_mutex_unlock(&(rdp_client->message_lock));
 
 }
