@@ -33,11 +33,21 @@
  */
 #define RDP_CLIENT_HOSTNAME_SIZE 32
 
+/**
+ * The default server response timeout, in seconds.
+ */
+#define RDP_DEFAULT_TIMEOUT 10
+
 /**
  * The default RDP port.
  */
 #define RDP_DEFAULT_PORT 3389
 
+/**
+ * The default SFTP connection timeout, in seconds.
+ */
+#define RDP_DEFAULT_SFTP_TIMEOUT 10
+
 /**
  * The default RDP port used by Hyper-V "VMConnect".
  */
@@ -58,6 +68,11 @@
  */
 #define RDP_DEFAULT_DEPTH  16 
 
+/**
+ * The color depth required by the RDPGFX channel, in bits.
+ */
+#define RDP_GFX_REQUIRED_DEPTH 32
+
 /**
  * The filename to use for the screen recording, if not specified.
  */
@@ -151,6 +166,11 @@ typedef struct guac_rdp_settings {
      */
     int port;
 
+    /**
+     * The timeout, in seconds, to wait for the remote host to respond.
+     */
+    int timeout;
+
     /**
      * The domain of the user logging in.
      */
@@ -281,6 +301,18 @@ typedef struct guac_rdp_settings {
      */
     int ignore_certificate;
 
+    /**
+     * Whether or not a certificate should be added to the local trust
+     * store on first use.
+     */
+    int certificate_tofu;
+
+    /**
+     * The fingerprints of host certificates that should be trusted for
+     * this connection.
+     */
+    char* certificate_fingerprints;
+
     /**
      * Whether authentication should be disabled. This is different from the
      * authentication that takes place when a user provides their username
@@ -435,6 +467,12 @@ typedef struct guac_rdp_settings {
      */
     char* sftp_port;
 
+    /**
+     * The number of seconds to attempt to connect to the SSH server before
+     * timing out.
+     */
+    int sftp_timeout;
+
     /**
      * The username to provide when authenticating with the SSH server for
      * SFTP.
@@ -459,6 +497,11 @@ typedef struct guac_rdp_settings {
      */
     char* sftp_passphrase;
 
+    /**
+     * The public key to use when connecting to the SFTP server, if applicable.
+     */
+    char* sftp_public_key;
+
     /**
      * The default location for file uploads within the SSH server. This will
      * apply only to uploads which do not use the filesystem guac_object (where
@@ -543,6 +586,12 @@ typedef struct guac_rdp_settings {
     int recording_include_keys;
 
     /**
+     * Non-zero if existing files should be appended to when creating a new 
+     * recording. Disabled by default.
+     */
+    int recording_write_existing;
+
+    /** 
      * The method to apply when the user's display changes size.
      */
     guac_rdp_resize_method resize_method;
@@ -552,6 +601,11 @@ typedef struct guac_rdp_settings {
      */
     int enable_audio_input;
 
+    /**
+     * Whether the RDP Graphics Pipeline Extension is enabled.
+     */
+    int enable_gfx;
+
     /**
      * Whether multi-touch support is enabled.
      */
