@@ -19,8 +19,6 @@
 
 #include "config.h"
 
-#include "common/cursor.h"
-#include "common/display.h"
 #include "ssh.h"
 #include "terminal/terminal.h"
 
