@@ -38,6 +38,7 @@ const char* GUAC_SSH_CLIENT_ARGS[] = {
     "hostname",
     "host-key",
     "port",
+    "timeout",
     "username",
     "password",
     GUAC_SSH_ARGV_FONT_NAME,
@@ -48,6 +49,7 @@ const char* GUAC_SSH_CLIENT_ARGS[] = {
     "sftp-disable-upload",
     "private-key",
     "passphrase",
+    "public-key",
 #ifdef ENABLE_SSH_AGENT
     "enable-agent",
 #endif
@@ -56,12 +58,14 @@ const char* GUAC_SSH_CLIENT_ARGS[] = {
     "typescript-path",
     "typescript-name",
     "create-typescript-path",
+    "typescript-write-existing",
     "recording-path",
     "recording-name",
     "recording-exclude-output",
     "recording-exclude-mouse",
     "recording-include-keys",
     "create-recording-path",
+    "recording-write-existing",
     "read-only",
     "server-alive-interval",
     "backspace",
@@ -96,6 +100,11 @@ enum SSH_ARGS_IDX {
      */
     IDX_PORT,
 
+    /**
+     * The timeout of the connection attempt, in seconds. Optional.
+     */
+    IDX_TIMEOUT,
+
     /**
      * The name of the user to login as. Optional.
      */
@@ -149,6 +158,11 @@ enum SSH_ARGS_IDX {
      */
     IDX_PASSPHRASE,
 
+    /**
+     * The public key to use for authentication, if any.
+     */
+    IDX_PUBLIC_KEY,
+
 #ifdef ENABLE_SSH_AGENT
     /**
      * Whether SSH agent forwarding support should be enabled.
@@ -191,6 +205,12 @@ enum SSH_ARGS_IDX {
      */
     IDX_CREATE_TYPESCRIPT_PATH,
 
+    /**
+     * Whether existing files should be appended to when creating a new
+     * typescript. Disabled by default.
+     */
+    IDX_TYPESCRIPT_WRITE_EXISTING,
+
     /**
      * The full absolute path to the directory in which screen recordings
      * should be written.
@@ -235,6 +255,12 @@ enum SSH_ARGS_IDX {
      */
     IDX_CREATE_RECORDING_PATH,
 
+    /**
+     * Whether existing files should be appended to when creating a new recording.
+     * Disabled by default.
+     */
+    IDX_RECORDING_WRITE_EXISTING,
+
     /**
      * "true" if this connection should be read-only (user input should be
      * dropped), "false" or blank otherwise.
@@ -374,6 +400,10 @@ guac_ssh_settings* guac_ssh_parse_args(guac_user* user,
         guac_user_parse_args_string(user, GUAC_SSH_CLIENT_ARGS, argv,
                 IDX_PASSPHRASE, NULL);
 
+    settings->public_key_base64 =
+        guac_user_parse_args_string(user, GUAC_SSH_CLIENT_ARGS, argv,
+                IDX_PUBLIC_KEY, NULL);
+
     /* Read maximum scrollback size */
     settings->max_scrollback =
         guac_user_parse_args_int(user, GUAC_SSH_CLIENT_ARGS, argv,
@@ -430,6 +460,11 @@ guac_ssh_settings* guac_ssh_parse_args(guac_user* user,
         guac_user_parse_args_string(user, GUAC_SSH_CLIENT_ARGS, argv,
                 IDX_PORT, GUAC_SSH_DEFAULT_PORT);
 
+    /* Parse the timeout value. */
+    settings->timeout =
+        guac_user_parse_args_int(user, GUAC_SSH_CLIENT_ARGS, argv,
+                IDX_TIMEOUT, GUAC_SSH_DEFAULT_TIMEOUT);
+
     /* Read-only mode */
     settings->read_only =
         guac_user_parse_args_boolean(user, GUAC_SSH_CLIENT_ARGS, argv,
@@ -455,6 +490,11 @@ guac_ssh_settings* guac_ssh_parse_args(guac_user* user,
         guac_user_parse_args_boolean(user, GUAC_SSH_CLIENT_ARGS, argv,
                 IDX_CREATE_TYPESCRIPT_PATH, false);
 
+    /* Parse allow write existing file flag */
+    settings->typescript_write_existing =
+        guac_user_parse_args_boolean(user, GUAC_SSH_CLIENT_ARGS, argv,
+                IDX_TYPESCRIPT_WRITE_EXISTING, false);
+
     /* Read recording path */
     settings->recording_path =
         guac_user_parse_args_string(user, GUAC_SSH_CLIENT_ARGS, argv,
@@ -485,6 +525,11 @@ guac_ssh_settings* guac_ssh_parse_args(guac_user* user,
         guac_user_parse_args_boolean(user, GUAC_SSH_CLIENT_ARGS, argv,
                 IDX_CREATE_RECORDING_PATH, false);
 
+    /* Parse allow write existing file flag */
+    settings->recording_write_existing =
+        guac_user_parse_args_boolean(user, GUAC_SSH_CLIENT_ARGS, argv,
+                IDX_RECORDING_WRITE_EXISTING, false);
+
     /* Parse server alive interval */
     settings->server_alive_interval =
         guac_user_parse_args_int(user, GUAC_SSH_CLIENT_ARGS, argv,
@@ -568,6 +613,7 @@ void guac_ssh_settings_free(guac_ssh_settings* settings) {
     guac_mem_free(settings->password);
     guac_mem_free(settings->key_base64);
     guac_mem_free(settings->key_passphrase);
+    guac_mem_free(settings->public_key_base64);
 
     /* Free display preferences */
     guac_mem_free(settings->font_name);
