@@ -32,6 +32,12 @@
  */
 #define GUAC_SSH_DEFAULT_PORT "22"
 
+/**
+ * The default number of seconds to attempt a connection to the SSH/SFTP
+ * server before giving up.
+ */
+#define GUAC_SSH_DEFAULT_TIMEOUT 10
+
 /**
  * The filename to use for the typescript, if not specified.
  */
@@ -69,6 +75,12 @@ typedef struct guac_ssh_settings {
      */
     char* port;
 
+    /**
+     * The number of seconds to attempt to connect to the SSH server before
+     * timing out.
+     */
+    int timeout;
+
     /**
      * The name of the user to login as, if any. If no username is specified,
      * this will be NULL.
@@ -93,6 +105,12 @@ typedef struct guac_ssh_settings {
      */
     char* key_passphrase;
 
+    /**
+     * The public key, encoded as base64, if any. If no public key is specified,
+     * this will be NULL.
+     */
+    char* public_key_base64;
+
     /**
      * Whether this connection is read-only, and user input should be dropped.
      */
@@ -202,6 +220,12 @@ typedef struct guac_ssh_settings {
      */
     bool create_typescript_path;
 
+    /**
+     * Whether existing files should be appended to when creating a new
+     * typescript. Disabled by default.
+     */
+    bool typescript_write_existing;
+
     /**
      * The path in which the screen recording should be saved, if enabled. If
      * no screen recording should be saved, this will be NULL.
@@ -245,6 +269,12 @@ typedef struct guac_ssh_settings {
      */
     bool recording_include_keys;
 
+    /**
+     * Whether existing files should be appended to when creating a new recording.
+     * Disabled by default.
+     */
+    bool recording_write_existing;
+
     /**
      * The number of seconds between sending server alive messages.
      */
