@@ -38,6 +38,7 @@
 const char* GUAC_TELNET_CLIENT_ARGS[] = {
     "hostname",
     "port",
+    "timeout",
     "username",
     "username-regex",
     "password",
@@ -48,12 +49,14 @@ const char* GUAC_TELNET_CLIENT_ARGS[] = {
     "typescript-path",
     "typescript-name",
     "create-typescript-path",
+    "typescript-write-existing",
     "recording-path",
     "recording-name",
     "recording-exclude-output",
     "recording-exclude-mouse",
     "recording-include-keys",
     "create-recording-path",
+    "recording-write-existing",
     "read-only",
     "backspace",
     "terminal-type",
@@ -82,6 +85,11 @@ enum TELNET_ARGS_IDX {
      */
     IDX_PORT,
 
+    /**
+     * The number of seconds to wait for the remote server to respond. Optional.
+     */
+    IDX_TIMEOUT,
+
     /**
      * The name of the user to login as. Optional.
      */
@@ -143,6 +151,12 @@ enum TELNET_ARGS_IDX {
      */
     IDX_CREATE_TYPESCRIPT_PATH,
 
+    /**
+     * Whether existing files should be appended to when creating a new
+     * typescript. Disabled by default.
+     */
+    IDX_TYPESCRIPT_WRITE_EXISTING,
+
     /**
      * The full absolute path to the directory in which screen recordings
      * should be written.
@@ -187,6 +201,12 @@ enum TELNET_ARGS_IDX {
      */
     IDX_CREATE_RECORDING_PATH,
 
+    /**
+     * Whether existing files should be appended to when creating a new recording.
+     * Disabled by default.
+     */
+    IDX_RECORDING_WRITE_EXISTING,
+
     /**
      * "true" if this connection should be read-only (user input should be
      * dropped), "false" or blank otherwise.
@@ -430,6 +450,11 @@ guac_telnet_settings* guac_telnet_parse_args(guac_user* user,
         guac_user_parse_args_string(user, GUAC_TELNET_CLIENT_ARGS, argv,
                 IDX_PORT, GUAC_TELNET_DEFAULT_PORT);
 
+    /* Read connection timeout */
+    settings->timeout =
+        guac_user_parse_args_int(user, GUAC_TELNET_CLIENT_ARGS, argv,
+                IDX_TIMEOUT, GUAC_TELNET_DEFAULT_TIMEOUT);
+
     /* Read typescript path */
     settings->typescript_path =
         guac_user_parse_args_string(user, GUAC_TELNET_CLIENT_ARGS, argv,
@@ -445,6 +470,11 @@ guac_telnet_settings* guac_telnet_parse_args(guac_user* user,
         guac_user_parse_args_boolean(user, GUAC_TELNET_CLIENT_ARGS, argv,
                 IDX_CREATE_TYPESCRIPT_PATH, false);
 
+    /* Parse allow write existing file flag */
+    settings->typescript_write_existing =
+        guac_user_parse_args_boolean(user, GUAC_TELNET_CLIENT_ARGS, argv,
+                IDX_TYPESCRIPT_WRITE_EXISTING, false);
+
     /* Read recording path */
     settings->recording_path =
         guac_user_parse_args_string(user, GUAC_TELNET_CLIENT_ARGS, argv,
@@ -475,6 +505,11 @@ guac_telnet_settings* guac_telnet_parse_args(guac_user* user,
         guac_user_parse_args_boolean(user, GUAC_TELNET_CLIENT_ARGS, argv,
                 IDX_CREATE_RECORDING_PATH, false);
 
+    /* Parse allow write existing file flag */
+    settings->recording_write_existing =
+        guac_user_parse_args_boolean(user, GUAC_TELNET_CLIENT_ARGS, argv,
+                IDX_RECORDING_WRITE_EXISTING, false);
+
     /* Parse backspace key code */
     settings->backspace =
         guac_user_parse_args_int(user, GUAC_TELNET_CLIENT_ARGS, argv,
