@@ -29,6 +29,11 @@
  */
 #define GUAC_VNC_DEFAULT_RECORDING_NAME "recording"
 
+/**
+ * The default number of seconds to attempt to connect to the SFTP server.
+ */
+#define GUAC_VNC_DEFAULT_SFTP_TIMEOUT 10
+
 /**
  * VNC-specific client data.
  */
@@ -54,6 +59,12 @@ typedef struct guac_vnc_settings {
      */
     char* password;
 
+    /**
+     * Disable the VNC client messages to request that the remote (server)
+     * display resize to match the client resolution.
+     */
+    bool disable_display_resize;
+
     /**
      * Space-separated list of encodings to use within the VNC session.
      */
@@ -82,6 +93,16 @@ typedef struct guac_vnc_settings {
      */
     bool lossless;
 
+    /**
+     * The level of compression to ask the VNC client library to perform.
+     */
+    int compress_level;
+
+     /**
+      * The quality level to ask the VNC client library to maintain.
+      */
+    int quality_level;
+
 #ifdef ENABLE_VNC_REPEATER
     /**
      * The VNC host to connect to, if using a repeater.
@@ -172,6 +193,11 @@ typedef struct guac_vnc_settings {
      */
     char* sftp_port;
 
+    /**
+     * The number of seconds to attempt to connect to the SFTP server.
+     */
+    int sftp_timeout;
+
     /**
      * The username to provide when authenticating with the SSH server for
      * SFTP.
@@ -196,6 +222,12 @@ typedef struct guac_vnc_settings {
      */
     char* sftp_passphrase;
 
+    /**
+     * The base64-encoded public key to use when authenticating with the SSH
+     * server for SFTP using key-based authentication.
+     */
+    char* sftp_public_key;
+
     /**
      * The default location for file uploads within the SSH server. This will
      * apply only to uploads which do not use the filesystem guac_object (where
@@ -274,6 +306,12 @@ typedef struct guac_vnc_settings {
      * as passwords, credit card numbers, etc.
      */
     bool recording_include_keys;
+
+    /**
+     * Whether existing files should be appended to when creating a new recording.
+     * Disabled by default.
+     */
+    bool recording_write_existing;
     
     /**
      * Whether or not to send the magic Wake-on-LAN (WoL) packet prior to
@@ -305,6 +343,11 @@ typedef struct guac_vnc_settings {
      */
     int wol_wait_time;
 
+    /**
+     * Whether or not to disable the input on the server side.
+     */
+    bool disable_server_input;
+
 } guac_vnc_settings;
 
 /**
