@@ -23,8 +23,6 @@
 #include "client.h"
 #include "clipboard.h"
 #include "common/clipboard.h"
-#include "common/cursor.h"
-#include "common/display.h"
 #include "cursor.h"
 #include "display.h"
 #include "log.h"
@@ -42,10 +40,14 @@
 #endif
 
 #include <guacamole/client.h>
+#include <guacamole/display.h>
+#include <guacamole/mem.h>
 #include <guacamole/protocol.h>
 #include <guacamole/recording.h>
 #include <guacamole/socket.h>
+#include <guacamole/string.h>
 #include <guacamole/timestamp.h>
+#include <guacamole/wol-constants.h>
 #include <guacamole/wol.h>
 #include <rfb/rfbclient.h>
 #include <rfb/rfbconfig.h>
@@ -137,6 +139,7 @@ rfbClient* guac_vnc_get_client(guac_client* client) {
 
     /* Framebuffer update handler */
     rfb_client->GotFrameBufferUpdate = guac_vnc_update;
+    vnc_client->rfb_GotCopyRect = rfb_client->GotCopyRect;
     rfb_client->GotCopyRect = guac_vnc_copyrect;
 
 #ifdef ENABLE_VNC_TLS_LOCKING
@@ -249,20 +252,97 @@ rfbClient* guac_vnc_get_client(guac_client* client) {
  *     The rfbClient to wait for.
  *
  * @param timeout
- *     The maximum amount of time to wait, in microseconds.
+ *     The maximum amount of time to wait, in milliseconds.
  *
  * @returns
  *     A positive value if data is available, zero if the timeout elapses
  *     before data becomes available, or a negative value on error.
  */
-static int guac_vnc_wait_for_messages(rfbClient* rfb_client, int timeout) {
+static int guac_vnc_wait_for_messages(rfbClient* rfb_client, int msec_timeout) {
 
     /* Do not explicitly wait while data is on the buffer */
     if (rfb_client->buffered)
         return 1;
 
     /* If no data on buffer, wait for data on socket */
-    return WaitForMessage(rfb_client, timeout);
+    return WaitForMessage(rfb_client, msec_timeout * 1000);
+
+}
+
+/**
+ * Handles any inbound VNC messages that have been received, updating the
+ * Guacamole display accordingly.
+ *
+ * @param vnc_client
+ *     The guac_vnc_client of the VNC connection whose current messages should
+ *     be handled.
+ *
+ * @return
+ *     True (non-zero) if messages were handled successfully, false (zero)
+ *     otherwise.
+ */
+static rfbBool guac_vnc_handle_messages(guac_client* client) {
+
+    guac_vnc_client* vnc_client = (guac_vnc_client*) client->data;
+    rfbClient* rfb_client = vnc_client->rfb_client;
+    guac_display_layer* default_layer = guac_display_default_layer(vnc_client->display);
+
+    /* All potential drawing operations must occur while holding an open context */
+    guac_display_layer_raw_context* context = guac_display_layer_open_raw(default_layer);
+    vnc_client->current_context = context;
+
+    /* Actually handle messages (this may result in drawing to the
+     * guac_display, resizing the display buffer, etc.) */
+    rfbBool retval = HandleRFBServerMessage(rfb_client);
+
+    /* Use the buffer of libvncclient directly if it matches the guac_display
+     * format */
+    unsigned int vnc_bpp = rfb_client->format.bitsPerPixel / 8;
+    if (vnc_bpp == GUAC_DISPLAY_LAYER_RAW_BPP && !vnc_client->settings->swap_red_blue) {
+
+        context->buffer = rfb_client->frameBuffer;
+        context->stride = guac_mem_ckd_mul_or_die(vnc_bpp, rfb_client->width);
+
+        /* Update bounds of pending frame to match those of RFB framebuffer */
+        guac_rect_init(&context->bounds, 0, 0, rfb_client->width, rfb_client->height);
+
+    }
+
+    /* There will be no further drawing operations */
+    guac_display_layer_close_raw(default_layer, context);
+    vnc_client->current_context = NULL;
+
+#ifdef LIBVNC_HAS_RESIZE_SUPPORT
+    // If screen was not previously initialized, check for it and set it.
+    if (!vnc_client->rfb_screen_initialized 
+            && rfb_client->screen.width > 0
+            && rfb_client->screen.height > 0) {
+        vnc_client->rfb_screen_initialized = true;
+        guac_client_log(client, GUAC_LOG_DEBUG, "Screen is now initialized.");
+    }
+
+    /*
+     * If the screen is now or has been initialized, check to see if the initial
+     * dimensions have already been sent. If not, and resize is not disabled,
+     * send the initial size.
+     */
+    if (vnc_client->rfb_screen_initialized) {
+        guac_vnc_settings* settings = vnc_client->settings;
+        if (!vnc_client->rfb_initial_resize && !settings->disable_display_resize) {
+            guac_client_log(client, GUAC_LOG_DEBUG,
+                    "Sending initial screen size to VNC server.");
+            guac_client_for_owner(client, guac_vnc_display_set_owner_size, rfb_client);
+            vnc_client->rfb_initial_resize = true;
+        }
+    }
+#endif // LIBVNC_HAS_RESIZE_SUPPORT
+
+    /* Resize the surface if VNC screen size has changed (this call
+     * automatically deals with invalid dimensions and is a no-op
+     * if the size has not changed) */
+    guac_display_layer_resize(default_layer, rfb_client->width, rfb_client->height);
+
+    return retval;
 
 }
 
@@ -274,17 +354,48 @@ void* guac_vnc_client_thread(void* data) {
 
     /* If Wake-on-LAN is enabled, attempt to wake. */
     if (settings->wol_send_packet) {
-        guac_client_log(client, GUAC_LOG_DEBUG, "Sending Wake-on-LAN packet, "
-                "and pausing for %d seconds.", settings->wol_wait_time);
-        
-        /* Send the Wake-on-LAN request. */
-        if (guac_wol_wake(settings->wol_mac_addr, settings->wol_broadcast_addr,
-                settings->wol_udp_port))
+
+        /**
+         * If wait time is set, send the wake packet and try to connect to the
+         * server, failing if the server does not respond.
+         */
+        if (settings->wol_wait_time > 0) {
+            guac_client_log(client, GUAC_LOG_DEBUG, "Sending Wake-on-LAN packet, "
+                    "and pausing for %d seconds.", settings->wol_wait_time);
+
+            /* char representation of a port should be, at most, 5 characters plus terminator. */
+            char* str_port = guac_mem_alloc(6);
+            if (guac_itoa(str_port, settings->port) < 1) {
+                guac_client_log(client, GUAC_LOG_ERROR, "Failed to convert port to integer for WOL function.");
+                guac_mem_free(str_port);
+                return NULL;
+            }
+
+            /* Send the Wake-on-LAN request and wait until the server is responsive. */
+            if (guac_wol_wake_and_wait(settings->wol_mac_addr,
+                    settings->wol_broadcast_addr,
+                    settings->wol_udp_port,
+                    settings->wol_wait_time,
+                    GUAC_WOL_DEFAULT_CONNECT_RETRIES,
+                    settings->hostname,
+                    (const char *) str_port,
+                    GUAC_WOL_DEFAULT_CONNECTION_TIMEOUT)) {
+                guac_client_log(client, GUAC_LOG_ERROR, "Failed to send WOL packet or connect to remote system.");
+                guac_mem_free(str_port);
+                return NULL;
+            }
+
+            guac_mem_free(str_port);
+
+        }
+
+        /* Just send the packet and continue the connection, or return if failed. */
+        else if(guac_wol_wake(settings->wol_mac_addr,
+                    settings->wol_broadcast_addr,
+                    settings->wol_udp_port)) {
+            guac_client_log(client, GUAC_LOG_ERROR, "Failed to send WOL packet.");
             return NULL;
-        
-        /* If wait time is specified, sleep for that amount of time. */
-        if (settings->wol_wait_time > 0)
-            guac_timestamp_msleep(settings->wol_wait_time * 1000);
+        }
     }
     
     /* Configure clipboard encoding */
@@ -363,6 +474,33 @@ void* guac_vnc_client_thread(void* data) {
                 return NULL;
             }
 
+            /* Import the public key, if that is specified. */
+            if (settings->sftp_public_key != NULL) {
+
+                guac_client_log(client, GUAC_LOG_DEBUG,
+                        "Attempting public key import");
+
+                /* Attempt to read public key */
+                if (guac_common_ssh_user_import_public_key(vnc_client->sftp_user,
+                            settings->sftp_public_key)) {
+
+                    /* Public key import fails. */
+                    guac_client_abort(client,
+                           GUAC_PROTOCOL_STATUS_CLIENT_UNAUTHORIZED,
+                           "Failed to import public key: %s",
+                            guac_common_ssh_key_error());
+
+                    guac_common_ssh_destroy_user(vnc_client->sftp_user);
+                    return NULL;
+
+                }
+
+                /* Success */
+                guac_client_log(client, GUAC_LOG_INFO,
+                        "Public key successfully imported.");
+
+            }
+
         }
 
         /* Otherwise, use specified password */
@@ -376,8 +514,8 @@ void* guac_vnc_client_thread(void* data) {
         /* Attempt SSH connection */
         vnc_client->sftp_session =
             guac_common_ssh_create_session(client, settings->sftp_hostname,
-                    settings->sftp_port, vnc_client->sftp_user, settings->sftp_server_alive_interval,
-                    settings->sftp_host_key, NULL);
+                    settings->sftp_port, vnc_client->sftp_user, settings->sftp_timeout,
+                    settings->sftp_server_alive_interval, settings->sftp_host_key, NULL);
 
         /* Fail if SSH connection does not succeed */
         if (vnc_client->sftp_session == NULL) {
@@ -416,6 +554,26 @@ void* guac_vnc_client_thread(void* data) {
     }
 #endif
 
+    /* Disable remote console (Server input) */
+    if (settings->disable_server_input) {
+        rfbSetServerInputMsg msg;
+        msg.type = rfbSetServerInput;
+        msg.status = 1;
+        msg.pad = 0;
+
+        /* Acquire lock for writing to server. */
+        pthread_mutex_lock(&(vnc_client->message_lock));
+
+        if (WriteToRFBServer(rfb_client, (char*)&msg, sz_rfbSetServerInputMsg))
+            guac_client_log(client, GUAC_LOG_DEBUG, "Successfully sent request to disable server input.");
+
+        else
+            guac_client_log(client, GUAC_LOG_WARNING, "Failed to send request to disable server input.");
+
+        /* Release lock. */
+        pthread_mutex_unlock(&(vnc_client->message_lock));
+    }
+
     /* Set remaining client data */
     vnc_client->rfb_client = rfb_client;
 
@@ -428,83 +586,60 @@ void* guac_vnc_client_thread(void* data) {
                 !settings->recording_exclude_output,
                 !settings->recording_exclude_mouse,
                 0, /* Touch events not supported */
-                settings->recording_include_keys);
+                settings->recording_include_keys,
+                settings->recording_write_existing);
     }
 
     /* Create display */
-    vnc_client->display = guac_common_display_alloc(client,
-            rfb_client->width, rfb_client->height);
+    vnc_client->display = guac_display_alloc(client);
+    guac_display_layer_resize(guac_display_default_layer(vnc_client->display), rfb_client->width, rfb_client->height);
 
     /* Use lossless compression only if requested (otherwise, use default
      * heuristics) */
-    guac_common_display_set_lossless(vnc_client->display, settings->lossless);
+    guac_display_layer_set_lossless(guac_display_default_layer(vnc_client->display),
+            settings->lossless);
+
+    /* If compression and display quality have been configured, set those. */
+    if (settings->compress_level >= 0 && settings->compress_level <= 9)
+        rfb_client->appData.compressLevel = settings->compress_level;
+
+    if (settings->quality_level >= 0 && settings->quality_level <= 9)
+        rfb_client->appData.qualityLevel = settings->quality_level;
 
     /* If not read-only, set an appropriate cursor */
     if (settings->read_only == 0) {
         if (settings->remote_cursor)
-            guac_common_cursor_set_dot(vnc_client->display->cursor);
+            guac_display_set_cursor(vnc_client->display, GUAC_DISPLAY_CURSOR_DOT);
         else
-            guac_common_cursor_set_pointer(vnc_client->display->cursor);
-
+            guac_display_set_cursor(vnc_client->display, GUAC_DISPLAY_CURSOR_POINTER);
     }
 
-    guac_socket_flush(client->socket);
+#ifdef LIBVNC_HAS_RESIZE_SUPPORT
+    /* Set initial state of the screen and resize flags. */
+    vnc_client->rfb_screen_initialized = false;
+    vnc_client->rfb_initial_resize = false;
+#endif // LIBVNC_HAS_RESIZE_SUPPORT
 
-    guac_timestamp last_frame_end = guac_timestamp_current();
+    guac_display_end_frame(vnc_client->display);
+
+    vnc_client->render_thread = guac_display_render_thread_create(vnc_client->display);
 
     /* Handle messages from VNC server while client is running */
     while (client->state == GUAC_CLIENT_RUNNING) {
 
-        /* Wait for start of frame */
-        int wait_result = guac_vnc_wait_for_messages(rfb_client,
-                GUAC_VNC_FRAME_START_TIMEOUT);
-        if (wait_result > 0) {
+        /* Wait for data and construct a reasonable frame */
+        int wait_result = guac_vnc_wait_for_messages(rfb_client, GUAC_VNC_MESSAGE_CHECK_INTERVAL);
+        while (wait_result > 0) {
 
-            int processing_lag = guac_client_get_processing_lag(client);
-            guac_timestamp frame_start = guac_timestamp_current();
+            /* Handle any message received */
+            if (!guac_vnc_handle_messages(client)) {
+                guac_client_abort(client,
+                        GUAC_PROTOCOL_STATUS_UPSTREAM_ERROR,
+                        "Error handling message from VNC server.");
+                break;
+            }
 
-            /* Read server messages until frame is built */
-            do {
-
-                guac_timestamp frame_end;
-                int frame_remaining;
-
-                /* Handle any message received */
-                if (!HandleRFBServerMessage(rfb_client)) {
-                    guac_client_abort(client,
-                            GUAC_PROTOCOL_STATUS_UPSTREAM_ERROR,
-                            "Error handling message from VNC server.");
-                    break;
-                }
-
-                /* Calculate time remaining in frame */
-                frame_end = guac_timestamp_current();
-                frame_remaining = frame_start + GUAC_VNC_FRAME_DURATION
-                                - frame_end;
-
-                /* Calculate time that client needs to catch up */
-                int time_elapsed = frame_end - last_frame_end;
-                int required_wait = processing_lag - time_elapsed;
-
-                /* Increase the duration of this frame if client is lagging */
-                if (required_wait > GUAC_VNC_FRAME_TIMEOUT)
-                    wait_result = guac_vnc_wait_for_messages(rfb_client,
-                            required_wait*1000);
-
-                /* Wait again if frame remaining */
-                else if (frame_remaining > 0)
-                    wait_result = guac_vnc_wait_for_messages(rfb_client,
-                            GUAC_VNC_FRAME_TIMEOUT*1000);
-                else
-                    break;
-
-            } while (wait_result > 0);
-
-            /* Record end of frame, excluding server-side rendering time (we
-             * assume server-side rendering time will be consistent between any
-             * two subsequent frames, and that this time should thus be
-             * excluded from the required wait period of the next frame). */
-            last_frame_end = frame_start;
+            wait_result = guac_vnc_wait_for_messages(rfb_client, 0);
 
         }
 
@@ -512,13 +647,12 @@ void* guac_vnc_client_thread(void* data) {
         if (wait_result < 0)
             guac_client_abort(client, GUAC_PROTOCOL_STATUS_UPSTREAM_ERROR, "Connection closed.");
 
-        /* Flush frame */
-        guac_common_surface_flush(vnc_client->display->default_surface);
-        guac_client_end_frame(client);
-        guac_socket_flush(client->socket);
-
     }
 
+    /* Stop render loop */
+    guac_display_render_thread_destroy(vnc_client->render_thread);
+    vnc_client->render_thread = NULL;
+
     /* Kill client and finish connection */
     guac_client_stop(client);
     guac_client_log(client, GUAC_LOG_INFO, "Internal VNC client disconnected");
