@@ -146,13 +146,14 @@ void guac_terminal_select_redraw(guac_terminal* terminal) {
 static int guac_terminal_find_char(guac_terminal* terminal,
         int row, int* column) {
 
+    guac_terminal_char* characters;
     int start_column = *column;
 
-    guac_terminal_buffer_row* buffer_row = guac_terminal_buffer_get_row(terminal->buffer, row, 0);
-    if (start_column < buffer_row->length) {
+    int length = guac_terminal_buffer_get_columns(terminal->current_buffer, &characters, NULL, row);
+    if (start_column >= 0 && start_column < length) {
 
         /* Find beginning of character */
-        guac_terminal_char* start_char = &(buffer_row->characters[start_column]);
+        guac_terminal_char* start_char = &(characters[start_column]);
         while (start_column > 0 && start_char->value == GUAC_CHAR_CONTINUATION) {
             start_char--;
             start_column--;
@@ -194,7 +195,7 @@ void guac_terminal_select_update(guac_terminal* terminal, int row, int column) {
 
     /* Only update if selection has changed */
     if (row != terminal->selection_end_row
-        || column <  terminal->selection_end_column
+        || column <=  terminal->selection_end_column
         || column >= terminal->selection_end_column + terminal->selection_end_width) {
 
         int width = guac_terminal_find_char(terminal, row, &column);
@@ -251,21 +252,21 @@ void guac_terminal_select_resume(guac_terminal* terminal, int row, int column) {
 }
 
 /**
- * Appends the text within the given subsection of a terminal row to the
+ * Appends the text within the given array of terminal characters to the
  * clipboard. The provided coordinates are considered inclusively (the
- * characters at the start and end column are included in the copied
- * text). Any out-of-bounds coordinates will be automatically clipped within
- * the bounds of the given row.
+ * characters at the start and end column are included in the copied text). Any
+ * out-of-bounds coordinates will be automatically clipped within the bounds of
+ * the given array.
  *
  * @param terminal
  *     The guac_terminal instance associated with the buffer containing the
  *     text being copied and the clipboard receiving the copied text.
  *
- * @param row
- *     The row number of the text within the terminal to be copied into the
- *     clipboard, where the first (top-most) row in the terminal is row 0. Rows
- *     within the scrollback buffer (above the top-most row of the terminal)
- *     will be negative.
+ * @param characters
+ *     The array of characters copied into the clipboard.
+ *
+ * @param length
+ *     The number of characters in the provided character array.
  *
  * @param start
  *     The first column of the text to be copied from the given row into the
@@ -275,38 +276,44 @@ void guac_terminal_select_resume(guac_terminal* terminal, int row, int column) {
  * @param end
  *     The last column of the text to be copied from the given row into the
  *     clipboard associated with the given terminal, where 0 is the first
- *     (left-most) column within the row, or a negative value to denote that
- *     the last column in the row should be used.
+ *     (left-most) column within the row.
  */
-static void guac_terminal_clipboard_append_row(guac_terminal* terminal,
-        int row, int start, int end) {
+static void guac_terminal_clipboard_append_characters(guac_terminal* terminal,
+        guac_terminal_char* characters, unsigned int length, int start, int end) {
 
     char buffer[1024];
-    int i = start;
-
-    guac_terminal_buffer_row* buffer_row =
-        guac_terminal_buffer_get_row(terminal->buffer, row, 0);
+    int eol;
 
     /* If selection is entirely outside the bounds of the row, then there is
      * nothing to append */
-    if (start < 0 || start > buffer_row->length - 1)
+    if (start < 0 || end < 0 || start >= length)
         return;
 
-    /* Clip given range to actual bounds of row */
-    if (end < 0 || end > buffer_row->length - 1)
-        end = buffer_row->length - 1;
+    /* Ensure desired end column is within bounds */
+    if (end >= length)
+        end = length - 1;
+
+    /* Get position of last not null char */
+    for (eol = end; eol > start; eol--) {
+        if (characters[eol].value != 0)
+            break;
+    }
 
     /* Repeatedly convert chunks of terminal buffer rows until entire specified
      * region has been appended to clipboard */
-    while (i <= end) {
+    for (int i = start; i <= end;) {
 
         int remaining = sizeof(buffer);
         char* current = buffer;
 
         /* Convert as many codepoints within the given range as possible */
-        for (i = start; i <= end; i++) {
+        for (; i <= end; i++) {
 
-            int codepoint = buffer_row->characters[i].value;
+            int codepoint = characters[i].value;
+
+            /* Fill empty with spaces if not at end of line */
+            if (codepoint == 0 && i < eol)
+                codepoint = GUAC_CHAR_SPACE;
 
             /* Ignore null (blank) characters */
             if (codepoint == 0 || codepoint == GUAC_CHAR_CONTINUATION)
@@ -351,29 +358,27 @@ void guac_terminal_select_end(guac_terminal* terminal) {
     guac_terminal_select_normalized_range(terminal,
             &start_row, &start_col, &end_row, &end_col);
 
-    /* If only one row, simply copy */
-    if (end_row == start_row)
-        guac_terminal_clipboard_append_row(terminal, start_row, start_col, end_col);
+    guac_terminal_char* characters;
+    bool last_row_was_wrapped = true;
 
-    /* Otherwise, copy multiple rows */
-    else {
+    for (int row = start_row; row <= end_row; row++) {
 
-        /* Store first row */
-        guac_terminal_clipboard_append_row(terminal, start_row, start_col, -1);
-
-        /* Store all middle rows */
-        for (int row = start_row + 1; row < end_row; row++) {
+        /* Add a newline only if the previous line was not wrapped */
+        if (!last_row_was_wrapped)
             guac_common_clipboard_append(terminal->clipboard, "\n", 1);
-            guac_terminal_clipboard_append_row(terminal, row, 0, -1);
-        }
 
-        /* Store last row */
-        guac_common_clipboard_append(terminal->clipboard, "\n", 1);
-        guac_terminal_clipboard_append_row(terminal, end_row, 0, end_col);
+        /* Append next row from desired region, adjusting the start/end column
+         * to account for selections that start or end in the middle of a row.
+         * With the exception of the start and end rows, all other rows are
+         * copied in their entirety. */
+        int length = guac_terminal_buffer_get_columns(terminal->current_buffer, &characters, &last_row_was_wrapped, row);
+        guac_terminal_clipboard_append_characters(terminal, characters, length,
+            (row == start_row) ? start_col : 0,
+            (row == end_row)   ? end_col   : length - 1);
 
     }
 
-    /* Send data */
+    /* Broadcast copied data to all connected users only if allowed */
     if (!terminal->disable_copy) {
         guac_common_clipboard_send(terminal->clipboard, client);
         guac_socket_flush(socket);
